package com.sxc.etaxincome.domain.command.query;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.api.config.annotation.NacosValue;
import com.google.common.base.Stopwatch;
import com.google.common.collect.Lists;
import com.sxc.etaxincome.common.util.RetryHttpHelper;
import com.sxc.etaxincome.domain.business.IApiConfigManager;
import com.sxc.etaxincome.domain.constants.EtaxReportConstants;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.common.CommonRequestWrapper;
import com.sxc.etaxincome.domain.model.common.CommonResponse;
import com.sxc.etaxincome.domain.model.common.PageList;
import com.sxc.etaxincome.domain.model.cookie.TokenNecessary;
import com.sxc.etaxincome.domain.model.report.*;
import com.sxc.etaxincome.domain.template.ResponseHandlerTemplate;
import com.sxc.etaxincome.enums.ApiCode;
import com.sxc.framework.common.httpclient.ResponseContent;
import com.sxc.framework.common.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.TimeUnit;

import static com.sxc.etaxincome.domain.constants.EtaxReportConstants.*;

@Slf4j
@Component(value = "queryReportListQueryPageListCommand")
public class QueryReportListQueryPageListCommand implements IQueryPageListCommand {

    @NacosValue(value = "${url.etax.report}", autoRefreshed = true)
    private String etaxReportUrl;

    @Autowired
    private IApiConfigManager apiConfigManager;

    @Override
    public CommonResponse<PageList<IReportResponse>> execute(CommonRequestWrapper commonRequestWrapper) {
        Stopwatch stopwatch = Stopwatch.createStarted();
        ApiConfig apiConfig = apiConfigManager.getApiConfig(ApiCode.QUERY_REPORT_PAGE_LIST.getCode());
        JSONObject requestDataJsonObject = new JSONObject();
        String requestDataStr = commonRequestWrapper.getData();
        String requestSerialNo = commonRequestWrapper.getRequestSerialNo();
        log.info("queryReportPageList requestSerialNo:{} data:{}", requestSerialNo, requestDataStr);
        QueryReportListRequestWrapper queryReportListRequestWrapper = JSON.parseObject(requestDataStr, QueryReportListRequestWrapper.class);
        TokenNecessary tokenNecessary = queryReportListRequestWrapper.decodeToken();
        String token = tokenNecessary.getToken();
        requestDataJsonObject.put(API_PARAM_TOKEN, token);
        requestDataJsonObject.put(API_PARAM_AREA_CODE, queryReportListRequestWrapper.getArea());
        String test = queryReportListRequestWrapper.getTest();
        if (StringUtils.isNotBlank(test)) {
            requestDataJsonObject.put(API_PARAM_TEST, test);
        }
        if (StringUtils.isNotBlank(queryReportListRequestWrapper.getReportDateStart())) {
            requestDataJsonObject.put("sbrqq", queryReportListRequestWrapper.getReportDateStart());
        }
        if (StringUtils.isNotBlank(queryReportListRequestWrapper.getReportDateEnd())) {
            requestDataJsonObject.put("sbrqz", queryReportListRequestWrapper.getReportDateEnd());
        }
        if (StringUtils.isNotBlank(queryReportListRequestWrapper.getDeclareDateStart())) {
            requestDataJsonObject.put("skssqq", queryReportListRequestWrapper.getDeclareDateStart());
        }
        if (StringUtils.isNotBlank(queryReportListRequestWrapper.getDeclareDateEnd())) {
            requestDataJsonObject.put("skssqz", queryReportListRequestWrapper.getDeclareDateEnd());
        }
        if (StringUtils.isNotBlank(queryReportListRequestWrapper.getCancelStatus())) {
            requestDataJsonObject.put("zfbz", queryReportListRequestWrapper.getCancelStatus());
        }
        if (StringUtils.isNotBlank(queryReportListRequestWrapper.getReportType())) {
            requestDataJsonObject.put("yzpzzlDm", queryReportListRequestWrapper.getReportType());
        }
        if (CollectionUtils.isNotEmpty(queryReportListRequestWrapper.getUpdateTypeList())) {
            requestDataJsonObject.put("gzlx", queryReportListRequestWrapper.getUpdateTypeList());
        }
        if (queryReportListRequestWrapper.getPageSize() != null) {
            requestDataJsonObject.put("pageSize", queryReportListRequestWrapper.getPageSize().toString());
        }
        if (queryReportListRequestWrapper.getPageNumber() != null) {
            requestDataJsonObject.put("pageNumber", queryReportListRequestWrapper.getPageNumber().toString());
        }
        try {
            String etaxReportUrlFull = etaxReportUrl + apiConfig.getApiUrl();
            log.info("url:{}", etaxReportUrlFull);
            String requestJson = JSONUtil.toString(requestDataJsonObject);
            log.info("request:{}", requestJson);
            ResponseContent responseContent = RetryHttpHelper.retryPostJsonEntity(etaxReportUrlFull, requestJson);
            String response = responseContent.getContent();
            log.info("response:{}", response);
            stopwatch.stop();
            log.info("query report page list elapsed time:{}ms", stopwatch.elapsed(TimeUnit.MILLISECONDS));
            return new ResponseHandlerTemplate<PageList<IReportResponse>>() {
                @Override
                public CommonResponse<PageList<IReportResponse>> handleSuccess(JSONObject jsonObject) {
                    JSONObject resultJsonObject = jsonObject.getJSONObject(EtaxReportConstants.ETAX_RESPONSE_DATA_KEY);
                    JSONArray jsonArray = resultJsonObject.getJSONArray("List");
                    Integer total = resultJsonObject.getInteger("Total");
                    Integer pageSize = resultJsonObject.getInteger("PageSize");
                    Integer pageNumber = resultJsonObject.getInteger("PageNumber");
                    List<IReportResponse> invoiceResponseList = Lists.newArrayListWithExpectedSize(10);
                    for (int i = 0; i < jsonArray.size(); i++) {
                        JSONObject jsonObjectInvoice = jsonArray.getJSONObject(i);
                        ReportResponse reportResponse = new ReportResponse();
                        reportResponse.putAll(jsonObjectInvoice);
                        invoiceResponseList.add(reportResponse);
                    }
                    PageList<IReportResponse> pageList = PageList.<IReportResponse>builder()
                            .total(total)
                            .pageNumber(pageNumber)
                            .pageSize(pageSize)
                            .list(invoiceResponseList).build();
                    return CommonResponse.success(pageList);
                }
            }.handle(response);
        } catch (Exception e) {
            log.error("query report page list error:", e);
            throw new EtaxIncomeException(e);
        }
    }
}
