package com.sxc.etaxincome.domain.support;

import com.google.common.collect.Lists;
import com.sxc.etaxincome.domain.exception.EtaxIncomeException;
import com.sxc.etaxincome.domain.model.common.DateTimeRange;
import com.sxc.framework.common.date.DateUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author caosongqing
 * @date 2023/1/11
 * @description
 */
public class DateProvider {
    private DateProvider() {

    }

    /**
     * 获取给定时间前一小时的时间
     * @param date
     * @return java.util.Date
     */
    public static Date beforeOneHour(Date date) {
        Calendar calendar = Calendar.getInstance();
        /*
        HOUR_OF_DAY 指示一天中的小时
        */
        calendar.setTime(date);
        calendar.add(Calendar.HOUR_OF_DAY, -1);
        return calendar.getTime();
    }

    /**
     * 获取给定时间后一小时的时间
     * @param date
     * @return java.util.Date
     */
    public static Date afterOneHour(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        /*
        HOUR_OF_DAY 指示一天中的小时
        */
        calendar.add(Calendar.HOUR_OF_DAY, 1);
        return calendar.getTime();
    }

    public static DateTimeRange getHourRange(Date hourTime) {
        String currentHourTimeStr = DateUtils.date2Str(hourTime, new SimpleDateFormat("yyyy-MM-dd HH"));
        return DateTimeRange.builder()
                .startDateTime(currentHourTimeStr + ":00:00")
                .endDateTime(currentHourTimeStr + ":59:59")
                .build();
    }

    public static DateTimeRange getLastHourRange(Date date) {
        Date lastHourTime = beforeOneHour(date);
        return getHourRange(lastHourTime);
    }

    public static DateTimeRange getNextHour(Date date) {
        Date nextHourTime = afterOneHour(date);
        return getHourRange(nextHourTime);
    }

    public static DateTimeRange today() {
        String today = DateUtils.getToday(DateUtils.getYmdDash());
        return renderDateRange(today, today);
    }

    public static DateTimeRange yesterday() {
        String yesterday = DateUtils.getYesterday();
        return renderDateRange(yesterday, yesterday);
    }

    public static DateTimeRange lastOneWeekUntilYesterday() {
        try {
            String yesterday = DateUtils.getYesterday();
            String dateStart = DateUtils.formatAddDate(yesterday, DateUtils.YMD_DASH, -6);
            String dateEnd = yesterday;
            return renderDateRange(dateStart, dateEnd);
        } catch (Exception e) {
            throw new EtaxIncomeException("get last one week until yesterday error");
        }
    }

    public static DateTimeRange pastNDays(int n) {
        try {
            String today = DateUtils.getToday(DateUtils.getYmdDash());
            String dateStart = DateUtils.formatAddDate(today, DateUtils.YMD_DASH, -1 * n);
            String dateEnd = DateUtils.getYesterday();
            return renderDateRange(dateStart, dateEnd);
        } catch (Exception e) {
            throw new EtaxIncomeException("get past n days error");
        }
    }

    public static DateTimeRange lastWeek() {
        try {
            Date mondayDate = thisWeekMonday();
            String mondayDateStr = DateUtils.date2Str(mondayDate, DateUtils.getYmdDash());
            String dateStart = DateUtils.formatAddDate(mondayDateStr, DateUtils.YMD_DASH, -7);
            String dateEnd = DateUtils.formatAddDate(mondayDateStr, DateUtils.YMD_DASH, -1);
            return renderDateRange(dateStart, dateEnd);
        } catch (Exception e) {
            throw new EtaxIncomeException("get last week error");
        }
    }

    /**
     * 获取当前日期的本周一是几号
     *
     * @return 本周一的日期
     */
    public static Date thisWeekMonday() {
        return thisWeekMonday(new Date());
    }

    public static Date thisWeekMonday(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        // 获得当前日期是一个星期的第几天
        // 获取的数表示的是每个星期的第几天，不能改变，其中星期日为第一天
        // 如果是星期日则获取天数时获取到的数字为1 在后面进行相减的时候出错
        int dayWeek = cal.get(Calendar.DAY_OF_WEEK);
        if (1 == dayWeek) {
            cal.add(Calendar.DAY_OF_MONTH, -1);
        }
        // 设置一个星期的第一天，按中国的习惯一个星期的第一天是星期一
        cal.setFirstDayOfWeek(Calendar.MONDAY);
        // 获得当前日期是一个星期的第几天
        int day = cal.get(Calendar.DAY_OF_WEEK);
        // 根据日历的规则，给当前日期减去星期几与一个星期第一天的差值
        // cal.getFirstDayOfWeek()根据前面的设置来动态的改变此值
        cal.add(Calendar.DATE, cal.getFirstDayOfWeek() - day);
        return cal.getTime();
    }

    public static String pastOneWeekDate() {
        String today = DateUtils.getToday(DateUtils.getYmdDash());
        return pastOneWeekDate(today);
    }

    public static String pastOneWeekDate(String dateStr) {
        try {
            return DateUtils.formatAddDate(dateStr, DateUtils.YMD_DASH, -7);
        } catch (ParseException e) {
            throw new EtaxIncomeException("get past monday error");
        }
    }

    public static String lastWeekMonday() {
        Date mondayDate = thisWeekMonday();
        String mondayDateStr = DateUtils.date2Str(mondayDate, DateUtils.getYmdDash());
        return pastOneWeekDate(mondayDateStr);
    }

    public static DateTimeRange lastMonth() {
        String dateStart = DateUtils.getPastMonthFirst();
        String dateEnd = DateUtils.getPastMonthLast();
        return renderDateRange(dateStart, dateEnd);
    }

    public static DateTimeRange lastQuarter() {
        String dateStart = DateUtils.getPastQuarterFirst();
        String dateEnd = DateUtils.getPastQuarterLast();
        return renderDateRange(dateStart, dateEnd);
    }

    public static DateTimeRange lastHalfAYear() {
        String dateStart = DateUtils.getPastHalfAYearFirst();
        String dateEnd = DateUtils.getPastHalfAYearLast();
        return renderDateRange(dateStart, dateEnd);
    }

    public static DateTimeRange lastYear() {
        int lastYear = DateUtils.getYear() - 1;
        String dateStart = DateUtils.formatDate(DateUtils.getYearFirst(lastYear), DateUtils.YMD_DASH);
        String dateEnd = DateUtils.formatDate(DateUtils.getYearLast(lastYear), DateUtils.YMD_DASH);
        return renderDateRange(dateStart, dateEnd);
    }

    public static String pastMonthFirst() {
        return DateUtils.getPastMonthFirst();
    }


    public static String pastYearFirst() {
        int lastYear = DateUtils.getYear() - 1;
        return DateUtils.formatDate(DateUtils.getYearFirst(lastYear), DateUtils.YMD_DASH);
    }

    public static DateTimeRange renderDateRange(String dateStart, String dateEnd) {
        String dateTimeStart = getDateTimeHead(dateStart);
        String dateTimeEnd = getDateTimeTail(dateEnd);
        return DateTimeRange.builder()
                .startDateTime(dateTimeStart)
                .endDateTime(dateTimeEnd)
                .build();
    }

    public static List<Date> hourTimeListYesterday() {
        String yesterdayStr = DateUtils.getYesterday();
        return hourTimeListDate(yesterdayStr);
    }

    public static List<Date> hourTimeListDate(String dateStr) {
        List<Date> dateList = Lists.newArrayListWithExpectedSize(10);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(DateUtils.str2Date(dateStr, DateUtils.getYmdDash()));
        for (int i = 0; i < 24; i++) {
            dateList.add((Date) calendar.getTime().clone());
            calendar.add(Calendar.HOUR_OF_DAY, 1);
        }
        return dateList;
    }

    public static List<Date> hourTimeListUntil(String fromDateStr, String untilDateStr) {
        Date fromDate = DateUtils.str2Date(fromDateStr, DateUtils.getYmdhmsDash());
        Date untilDate = DateUtils.str2Date(untilDateStr, DateUtils.getYmdhmsDash());
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fromDate);
        Calendar untilCalendar = Calendar.getInstance();
        untilCalendar.setTime(untilDate);
        List<Date> dateList = Lists.newArrayListWithExpectedSize(10);
        while(calendar.before(untilCalendar)) {
            dateList.add((Date) calendar.getTime().clone());
            calendar.add(Calendar.HOUR_OF_DAY, 1);
        }
        return dateList;
    }

    public static List<Date> mondayListUntil(String fromDateStr, String untilDateStr) {
        Date fromDate = DateUtils.str2Date(fromDateStr, DateUtils.getYmdDash());
        Date untilDate = DateUtils.str2Date(untilDateStr, DateUtils.getYmdDash());
        Date firstMondayDate = firstMondayAfter(fromDate);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(firstMondayDate);
        Calendar untilCalendar = Calendar.getInstance();
        untilCalendar.setTime(untilDate);
        List<Date> dateList = Lists.newArrayListWithExpectedSize(10);
        while(calendar.before(untilCalendar)) {
            dateList.add((Date) calendar.getTime().clone());
            calendar.add(Calendar.DAY_OF_MONTH, 7);
        }
        return dateList;
    }

    public static DateTimeRange oneWeekDateRange(Date startDate) {
        String statDateStr = DateUtils.date2Str(startDate, DateUtils.getYmdDash());
        String toDateStr = DateUtils.date2Str(DateUtils.addDate(startDate, 6), DateUtils.getYmdDash());
        return DateProvider.renderDateRange(statDateStr, toDateStr);
    }

    public static Date firstMondayAfter(Date fromDate) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fromDate);
        while(calendar.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY) {
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        return calendar.getTime();
    }

    public static DateTimeRange renderDateRange(String date) {
        return renderDateRange(date, date);
    }

    public static List<String> dateListBetween(String from, String to) {
        Date fromDate = DateUtils.str2Date(from, DateUtils.getYmdDash());
        Date untilDate = DateUtils.str2Date(to, DateUtils.getYmdDash());
        if (fromDate.after(untilDate)) {
            return Collections.emptyList();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fromDate);
        Calendar untilCalendar = Calendar.getInstance();
        untilCalendar.setTime(untilDate);
        List<String> dateList = Lists.newArrayListWithExpectedSize(10);
        while(!calendar.after(untilCalendar)) {
            dateList.add(DateUtils.formatDate((Date) calendar.getTime().clone(), DateUtils.YMD_DASH));
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        return dateList;
    }

    public static String formatDateRange(DateTimeRange dateTimeRange) {
        String startDateTime = dateTimeRange.getStartDateTime();
        String endDateTime = dateTimeRange.getEndDateTime();
        String startDate = startDateTime.substring(0, 10);
        String endDate = endDateTime.substring(0, 10);
        if (startDate.equals(endDate)) {
            return startDate;
        } else {
            return startDateTime + "至" + endDateTime;
        }
    }

    public static String getDateTimeHead(String date) {
        return date + " 00:00:00";
    }

    public static String getDateTimeTail(String date) {
        return date + " 23:59:59";
    }

    public static String getLongDateTimeHead(String date) {
        if(date.contains("-") && !date.contains(":")) {
            return getDateTimeHead(date);
        }
        return date;
    }

    public static String getLongDateTimeTail(String date) {
        if(date.contains("-") && !date.contains(":")) {
            return getDateTimeTail(date);
        }
        return date;
    }

}
